@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 James Aylett
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006,2007,2008,2009,2010,2011 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007,2008,2009,2010,2011,2014 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,6 +40,7 @@
 #include "cgiparam.h"
 #include "query.h"
 #include "str.h"
+#include "expand.h"
 
 using namespace std;
 
@@ -203,7 +204,9 @@ try {
 	    tmprset.add_document(docid);
 
 	    OmegaExpandDecider decider(db);
-	    Xapian::ESet eset(enquire->get_eset(40, tmprset, &decider));
+	    set_expansion_scheme(*enquire, option);
+	    Xapian::ESet eset(enquire->get_eset(40, tmprset, 0,
+						expand_param_k, &decider));
 	    for (Xapian::ESetIterator i = eset.begin(); i != eset.end(); i++) {
 		if (!query_string.empty()) query_string += ' ';
 		query_string += pretty_term(*i);
