@@ -70,7 +70,7 @@
 #include "gnu_getopt.h"
 
 #ifndef HAVE_MKDTEMP
-extern char * mkdtemp(char *);
+#include "portability/mkdtemp.h"
 #endif
 
 using namespace std;
@@ -86,6 +86,7 @@ static bool follow_symlinks = false;
 static bool ignore_exclusions = false;
 static bool spelling = false;
 static off_t  max_size = 0;
+static std::string pretty_max_size;
 static bool verbose = false;
 static enum {
     EMPTY_BODY_WARN, EMPTY_BODY_INDEX, EMPTY_BODY_SKIP
@@ -335,7 +336,8 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
     }
 
     if (max_size > 0 && d.get_size() > max_size) {
-	skip(file, "Larger than size limit", SKIP_VERBOSE_ONLY);
+	skip(file, "Larger than size limit of " + pretty_max_size,
+	     SKIP_VERBOSE_ONLY);
 	return;
     }
 
@@ -1411,6 +1413,23 @@ main(int argc, char **argv)
 	    off_t size = parse_size(optarg);
 	    if (size >= 0) {
 		max_size = size;
+		const char * suffix;
+		// Set lsb to the lowest set bit in max_size.
+		off_t lsb = max_size & -max_size;
+		if (lsb >= off_t(1L << 30)) {
+		    size >>= 30;
+		    suffix = "GB";
+		} else if (lsb >= off_t(1L << 20)) {
+		    size >>= 20;
+		    suffix = "MB";
+		} else if (lsb >= off_t(1L << 10)) {
+		    size >>= 10;
+		    suffix = "KB";
+		} else {
+		    suffix = "B";
+		}
+		pretty_max_size = str(size);
+		pretty_max_size += suffix;
 		break;
 	    }
 	    cerr << PROG_NAME": bad max size '" << optarg << "'" << endl;
@@ -1434,36 +1453,6 @@ main(int argc, char **argv)
     if (!endswith(baseurl, '/')) {
 	baseurl += '/';
     }
-
-    if (optind >= argc || optind + 2 < argc) {
-	cerr << PROG_NAME": you must specify a directory to index.\n"
-"Do this either as a single directory (corresponding to the base URL)\n"
-"or two directories - the first corresponding to the base URL and the second\n"
-"a subdirectory of that to index." << endl;
-	return 1;
-    }
-    root = argv[optind];
-    if (!endswith(root, '/')) {
-	root += '/';
-    }
-    string start_url;
-    if (optind + 2 == argc) {
-	start_url = argv[optind + 1];
-	if (startswith(start_url, '/')) {
-	    // Make relative to root.
-	    if (!startswith(start_url, root)) {
-		cerr << PROG_NAME": '" << argv[optind + 1] << "' "
-		    "is not a subdirectory of '" << argv[optind] << "'."
-		     << endl;
-		return 1;
-	    }
-	    start_url.erase(0, root.size());
-	}
-	if (!endswith(start_url, '/')) {
-	    start_url += '/';
-	}
-    }
-
     string::size_type j;
     j = find_if(baseurl.begin(), baseurl.end(), p_notalnum) - baseurl.begin();
     if (j > 0 && baseurl.substr(j, 3) == "://") {
@@ -1477,7 +1466,8 @@ main(int argc, char **argv)
 	} else {
 	    // Path:
 	    string::size_type path_len = baseurl.size() - k;
-	    // Subtract one to lose the trailing /, unless it's the initial / too.
+	    // Subtract one to lose the trailing /, unless it's the initial /
+	    // too.
 	    if (path_len > 1) --path_len;
 	    site_term = "P" + baseurl.substr(k, path_len);
 	    // Host:
@@ -1494,6 +1484,37 @@ main(int argc, char **argv)
 	site_term = "P" + baseurl.substr(0, path_len);
     }
 
+    if (optind >= argc || optind + 2 < argc) {
+	cerr << PROG_NAME": you must specify a directory to index.\n"
+"Do this either as a single directory (corresponding to the base URL)\n"
+"or two directories - the first corresponding to the base URL and the second\n"
+"a subdirectory of that to index." << endl;
+	return 1;
+    }
+
+    root = argv[optind];
+    if (!endswith(root, '/')) {
+	root += '/';
+    }
+    if (optind + 2 == argc) {
+	string start_url = argv[optind + 1];
+	if (startswith(start_url, '/')) {
+	    // Make relative to root.
+	    if (!startswith(start_url, root)) {
+		cerr << PROG_NAME": '" << argv[optind + 1] << "' "
+		    "is not a subdirectory of '" << argv[optind] << "'."
+		     << endl;
+		return 1;
+	    }
+	    start_url.erase(0, root.size());
+	}
+	if (!endswith(start_url, '/')) {
+	    start_url += '/';
+	}
+	root += start_url;
+	url_encode_path_lite(baseurl, start_url);
+    }
+
     int exitcode = 1;
     try {
 	if (!overwrite) {
@@ -1522,7 +1543,7 @@ main(int argc, char **argv)
 	}
 	indexer.set_stemmer(stemmer);
 
-	index_directory(root + start_url, baseurl + start_url, depth_limit, mime_map, sample_size);
+	index_directory(root, baseurl, depth_limit, mime_map, sample_size);
 	if (delete_removed_documents && old_docs_not_seen) {
 	    if (verbose) {
 		cout << "Deleting " << old_docs_not_seen << " old documents which weren't found" << endl;
